
#include <fstream>
#include <iostream>

#include "main.hpp"

int main(int argc, char **argv) {

  /* Parsing CLI Arguments */
  po::options_description desc("Available options");
  desc.add_options()("help,h", "show help")("verbose,v", "more verbose output")(
      "vert", po::value<string>()->required(), "vertex shader file")(
      "frag", po::value<string>()->required(), "fragment shader file");

  /* unused for now */
  po::positional_options_description pdesc;
  // NOTE: if you want to add a positional argument,
  // you need to create and option for it as it seems.
  pdesc.add("vert", 1);
  pdesc.add("frag", 1);

  po::variables_map vm;
  try {

    po::store(po::command_line_parser(argc, argv)
                  .options(desc)     // load options
                  .positional(pdesc) // load positionals
                  .run(),
              vm);
    po::notify(vm);
    if (vm.count("help") || argc < 3) {
      help(argv[0], desc);
      return 1;
    }
  } catch (po::error &e) {
    cout << e.what() << endl << endl;
    help(argv[0], desc);
    return EXIT_USAGE;
  }

  if (vm.count("verbose")) {
    printf("verbose!");
  }

  /* setup the shader program */
  ifstream vertexFileStream(vm["vert"].as<string>());
  ifstream fragFileStream(vm["frag"].as<string>());
  std::stringstream vertexSource;
  std::stringstream fragSource;
  vertexSource << vertexFileStream.rdbuf();
  fragSource << fragFileStream.rdbuf();

  if (vertexFileStream.fail()) {
    cout << "Error while reading the vertex shader file." << endl;
    return EXIT_IO;
  }
  if (fragFileStream.fail()) {
    cout << "Error while reading the frag shader file." << endl;
    return EXIT_IO;
  }
  vertexFileStream.close();
  fragFileStream.close();

  ShaderProgram shaderProgram(ShaderProgram::Base::Empty, vertexSource.str(),
                              fragSource.str());

  /* Run our main program */
  int result = mainWindow(&shaderProgram);

  return result;
}

void help(char *prog, po::options_description desc) {
  cout << "Usage:" << endl
       << prog
       << " [-vh] [--vert] shaders/vertex.glsl [--frag] shaders/fragment.glsl"
       << endl
       << endl
       << "return values:" << endl
       << "0\tSuccess"
       << "1\tGeneric Failure"
       << "2\tBad arguments"
       << "3\tI/O Error" << endl
       << endl
       << desc << endl;
  // NOTE: constants for the error codes are defined in main.hpp
}
